// origin source from:
// https://www.icp.uni-stuttgart.de/~hilfer/lehre/100-online/skriptum/html_book00/node76.html

/*********************************************************************
  zupf.cc -- Modellierung einer gezupften Saite
  Autor    : R. Hilfer
  Zweck    : Beispiel zur Vorlesung, demonstriert Karplus-Strong Alg
  Gebrauch : Eingabe von stdin:
               frq = Frequenz in Hertz
               t_max = Gesamtzeit
   Version  : 1.0 (Originalversion) erstellt am  12.10.2000
  -- output = Textfile, then plot using gnuplot
  --- ------------------------------------------------------------
   modified for output on soundcard (linux)           myr 1.4.2014
   g++ -Wall -o zupf zupf.cpp
   usage: 
   ./zupf      then input from stdin
   or
   ./zupf frequency[Hz] duration[sec]      e.g. ./zupf 440 5
   -- output = soundcard
  --- ------------------------------------------------------------
  renamed to propPluckedString.c                     myr 16.5.2019
  modified for running on Propeller Demo Board  ---- Audio Jack
  C++ -> C  change 'new' operator to malloc / free
  remove the stuff for soundcard
  exchange the double array to short array
  compilied with SimpleIDE  C/Memorymodel LMM                     
  -- output = DAC on Prop Demoboard
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h> 
#include <propeller.h>


#define P10 1<<10
#define CRTAMODE (1<<28) + (1<<27) + 10  // DUTY SINGLE ENDED
#define P11 1<<11
#define CRTBMODE (1<<28) + (1<<27) + 11  // DUTY SINGLE ENDED
#define u_0  28000
#define dt  1./44000.
#define GAIN 10
#define SPEED 2.

#define STACKSIZE 100
static int cogstack[STACKSIZE];

void pluck (void);
void start_lmm(void *func, int *stack_top );
void setRegister(int ID,short frq, double t_max );
typedef struct
{
  short frq;
  double t_max;
  short len;
  short plckpt;
  short u[500];
  unsigned char process_ready;
  unsigned char setup_ready;
} _controlregister;
volatile _controlregister controlregister[8];

int main (void)
{
  controlregister[1].setup_ready = 0;  
  //controlregister[2].setup_ready = 0;  
  //controlregister[3].setup_ready = 0;  
  //controlregister[4].setup_ready = 0;  
  //controlregister[5].setup_ready = 0;  
  //controlregister[6].setup_ready = 0; 
  //controlregister[7].setup_ready = 0; 
 
  start_lmm(pluck, cogstack + 1*STACKSIZE );
  start_lmm(pluck, cogstack + 2*STACKSIZE );
  //start_lmm(pluck, cogstack + 2*STACKSIZE );
  //start_lmm(pluck, cogstack + 3*STACKSIZE );
  //start_lmm(pluck, cogstack + 4*STACKSIZE );
  //start_lmm(pluck, cogstack + 5*STACKSIZE );
  //start_lmm(pluck, cogstack + 6*STACKSIZE );

L0:
  setRegister(1,440,1.0);
  //setRegister(1,220,0.5);
  setRegister(2,220,1.0);
//goto L0;
     
	return 0;
}

void pluck (void)
{
  
	short i;
	short uprv;
	double t = 0;
  unsigned myID = __builtin_propeller_cogid() ;
   
   DIRA |= P10; 
   DIRA |= P11;                           
   CTRA = CRTAMODE;
   CTRB = CRTBMODE;  

LOOP:

  //while (controlregister[myID].setup_ready == 0);	
	controlregister[myID].process_ready = 0;	
  // Initialize the String
   for( i=0; i<=controlregister[myID].plckpt; i++ )
   {     
     controlregister[myID].u[i] = 
     2 * u_0 * i/controlregister[myID].plckpt - u_0;
   }
   for( i=controlregister[myID].len-1; i>controlregister[myID].plckpt; i-- )
   {
     controlregister[myID].u[i]=
     2 * u_0 * (controlregister[myID].len-1-i)/
    (controlregister[myID].len-controlregister[myID].plckpt) - u_0;
   }

  // process .............................................................
   i=0;
   t=0;
   uprv=controlregister[myID].u[0];
   while( t < controlregister[myID].t_max)
   {
     controlregister[myID].u[i] = (controlregister[myID].u[i] >> 1) + (uprv >> 1);  // Lowpass = smoothing mean
     uprv = controlregister[myID].u[i];                       // z^(-1)
     
     
     FRQA = 0x80000000 + ((int)controlregister[myID].u[i] << GAIN);
     FRQB = FRQA;

     t = t + SPEED * dt;        // inc time  -- faster--     myr
     if(i==controlregister[myID].len-1) i=-1;         // wave reflection
     i++; 						       // next index
                         
   }

  
   // ....................................................... process end
  controlregister[myID].process_ready = 1;
  goto LOOP;
}

void setRegister(int ID,short frq, double t_max)
{
  controlregister[ID].setup_ready = 0;
  controlregister[ID].frq = frq;
  controlregister[ID].t_max = t_max;
  controlregister[ID].frq = controlregister[ID].frq << 1;
	controlregister[ID].len = 44000/controlregister[ID].frq; 
	controlregister[ID].plckpt = controlregister[ID].len >> 3; 
  controlregister[ID].setup_ready = 1; 
  //while(controlregister[ID].process_ready == 0);
}

void start_lmm(void *func, int *stack_top )
{
    extern unsigned int _load_start_kernel[];

    /* push the code address onto the stack */
    --stack_top;
    *stack_top = (int)func;

    /* now start the kernel */
    cognew(_load_start_kernel, stack_top);
}