// origin source from:
// https://www.icp.uni-stuttgart.de/~hilfer/lehre/100-online/skriptum/html_book00/node76.html

/*********************************************************************
  zupf.cc -- Modellierung einer gezupften Saite
  Autor    : R. Hilfer
  Zweck    : Beispiel zur Vorlesung, demonstriert Karplus-Strong Alg
  Gebrauch : Eingabe von stdin:
               frq = Frequenz in Hertz
               t_max = Gesamtzeit
   Version  : 1.0 (Originalversion) erstellt am  12.10.2000
  -- output = Textfile, then plot using gnuplot
  --- ------------------------------------------------------------
   modified for output on soundcard (linux)           myr 1.4.2014
   g++ -Wall -o zupf zupf.cpp
   usage: 
   ./zupf      then input from stdin
   or
   ./zupf frequency[Hz] duration[sec]      e.g. ./zupf 440 5
   -- output = soundcard
  --- ------------------------------------------------------------
  renamed to propPluckedString.c                     myr 16.5.2019
  modified for running on Propeller Demo Board  ---- Audio Jack
  C++ -> C  change 'new' operator to malloc / free
  remove the stuff for soundcard
  exchange the double array to short array
  compilied with SimpleIDE  C/Memorymodel LMM                     
  -- output = DAC on Prop Demoboard
  --- ------------------------------------------------------------
  small modifications                                myr 17.5.2019
  The generated frequency was only half the size of the desired one.
  fixed: 	line 79 frq = frq << 1;
  The readout speed and the Gain are now #defines
  #define GAIN 10
  #define SPEED 2.0

 *********************************************************************/

#include <stdio.h>
#include <stdlib.h> 
#include <propeller.h>

#define P10 1<<10
#define CRTAMODE (1<<28) + (1<<27) + 10  // DUTY SINGLE ENDED
#define P11 1<<11
#define CRTBMODE (1<<28) + (1<<27) + 11  // DUTY SINGLE ENDED
#define u_0  28000
#define dt  1./44100.
#define GAIN 10
#define SPEED 2.0

short pluck (short frq, double duration);
void ex_1 (void);
void ex_2 (void);


int main (void)
{
   DIRA |= P10; 
   DIRA |= P11;                           
   CTRA = CRTAMODE;
   CTRB = CRTBMODE;  

   ex_1(); 
   ex_2();
  // for measure with Spectrum Analyzer
  while(1)
  {
    if(pluck(440,2.0))    
      if(pluck(220,1.0))
          ;
  }
	return 0;
}

short pluck (short frq, double t_max)
{
	short *u = NULL;
	short len = 0;
	short plckpt = 0;
	short i,j;
	short uprv;
	double t = 0;
	frq = frq << 1;
	len = 44100/frq; 
	plckpt = len >> 3;  
	u = (short * ) malloc (len * sizeof(short));
	
	// Initialize the String
   for( i=0; i<=plckpt; i++ )
   {     
     u[i]=2 * u_0 * i/plckpt - u_0;
   }
   for( i=len-1; i>plckpt; i-- )
   {
     u[i]=2 * u_0 * (len-1-i)/(len-plckpt) - u_0;
   }

  // process ...
   i=0;
   //j=0;
   uprv=u[0];
   while( t < t_max)
   {
     u[i] = (u[i] >> 1) + (uprv >> 1);  // Lowpass = smoothing mean
     uprv = u[i];                       // z^(-1)
     
     //printf("%d\n",u[i]);
     FRQA = 0x80000000 + ((int)u[i] << GAIN);
     FRQB = FRQA >> 1;

     t = t + SPEED * dt;           // inc time  -- faster--     myr
     if(i==len-1) i=-1;         // wave reflection
     i++; 						       // next index
     //j++; 						       // count all samples , needed for soundcard                    
   }

	free(u);
	return 1;
}


void ex_1 (void)
{
  int ready = -1;
  pluck (330, 0.33);
  pluck (330, 0.33);
  pluck (330, 0.66);

  pluck (330, 0.33);
  pluck (330, 0.33);
  pluck (330, 0.66);


  pluck (330, 0.33);
  pluck (392, 0.33);
  pluck (262, 0.33);
  pluck (294, 0.33);
  pluck (330, 0.66);

  pluck (349, 0.33);
  pluck (349, 0.33);
  pluck (349, 0.33);

  pluck (349, 0.33);
  pluck (349, 0.33);
  pluck (330, 0.33);

  pluck (330, 0.33);
  pluck (330, 0.33);

  pluck (330, 0.33);
  pluck (294, 0.33);
  pluck (294, 0.33);
  pluck (330, 0.33);

  pluck (294, 0.66);
  pluck (392, 0.66);

  pluck (330, 0.33);
  pluck (330, 0.33);
  pluck (330, 0.66);

  pluck (330, 0.33);
  pluck (330, 0.33);
  pluck (330, 0.66);


  pluck (330, 0.33);
  pluck (392, 0.33);
  pluck (262, 0.33);
  pluck (294, 0.33);
  pluck (330, 0.66);

  pluck (349, 0.33);
  pluck (349, 0.33);
  pluck (349, 0.33);

  pluck (349, 0.33);
  pluck (349, 0.33);
  pluck (330, 0.33);

  pluck (330, 0.33);
  pluck (330, 0.33);

  pluck (330, 0.33);
  pluck (294, 0.33);
  pluck (294, 0.33);
  pluck (330, 0.33);

  ready = pluck (294, 0.8);
}

void ex_2 (void)
{
  pluck (262,0.5);
  pluck (294,0.5);
  pluck (330,0.5);
  pluck (350,0.5);
  pluck (393,0.5);
  pluck (440,0.5);
  pluck (495,0.5);
  pluck (524,0.5);
}